/*****************************************************************************
Author: Peter J. Vis
First Written: 8 Dec 2004
Last Updated: 12 Dec 2006
Title: Displaying text on a 128x64 LCD - Library Functions

Microcontroller: ATmega32
Crystal: 16Mhz
Platform: Development System

Documentation:
Further information regarding this code is available on my site:
Also check the library file for the Bar graph project. I have
provided a lot of comments on that.
http://petervis.com

PURPOSE:
This is the library file for displaying text on the LCD.

LIMITATIONS:
No part of this work may be used in commercial
applications without prior written permission. This work cannot be
used by Bloggers, or any kind of on-line publishing.

COPYRIGHT:	
All the material including function names are copyright protected.

CIRCUIT:
My Modular Development System

*******************************************************************************/

// My functions rely on the following
// support files. Make sure they are present
// in your compiler environment.

#include "128x64-text.h"
#include "font_data.h" 
#include <inttypes.h>
#include <util/delay.h>


void lcdInit (void)
{  
	
	cPort_ddr = 0xff; 
	cPort = 0x00; 

	dPort_ddr = 0xff;
	dPort = 0x00;  

	cPort |= (1<<lcdrst); 
 
	WriteCommand(DISP_OFF);	
	WriteCommand(START_LINE);	
	WriteCommand(DISP_ON);  
  
	clearl(); 
}




void setNewPage (unsigned char PageData)
{		
		PageData = PageData + SET_PAGE;
        WriteCommand(PageData);	
		
		WriteCommand(SET_ADDRESS);
}




void clearl (void){
 
 unsigned int i;
 
 column = 0;
 page = 0;
  
 WriteCommand(SET_ADDRESS);
 WriteCommand(SET_PAGE);
 
 
 for(i = 0; i < 1024 ;i++)
 {
  
  if (column == 128)
  {
   column = 0;
   page++;
   setNewPage(page);
  }

  cSelect(column);						
										
	DataMode(); 						
	dPort = 0b00000000; 
	LocknLoad();						

  column++;
 }
 
 column = 0;
 page = 0;
 
 WriteCommand(SET_ADDRESS);	
 WriteCommand(SET_PAGE);
 
}




void DataMode (void)
{   
  cPort |= (1<<lcdrs);
  cPort &= ~(1<<lcdrw); 
}




void WriteCommand(uint8_t cmd) {

		cPort |= (1<<lcdcs1);
        cPort |= (1<<lcdcs2);   
        		
        cPort &= ~(1<<lcdrw);  
		cPort &= ~(1<<lcdrs);   
		
    dPort = cmd;
    LocknLoad();
}




void cSelect(uint8_t col)
{

if (col <= 63){
	cPort |= (1<<lcdcs1);
	cPort &= ~(1<<lcdcs2);
	}
	
	if (col >= 64){
	cPort |= (1<<lcdcs2);
	cPort &= ~(1<<lcdcs1);
	}

}




void LocknLoad (void)
{
    cPort |= (1<<lcde);

   _delay_us(4);
   
    cPort &= ~(1<<lcde);
	
   _delay_us(4);
}




void setXY(uint8_t x, uint8_t y) {

	uint8_t cmd;
	
	if(x > 127) x = 0;
	if(y > 63)  y = 0;								
	
	page = y/8;
	column = x;
	cSelect(column); 
	
	cmd = SET_ADDRESS | x;	
	WriteCommand(cmd);
	
	cmd = SET_PAGE | page;
	WriteCommand(cmd);

}




void lcd_writech(uint8_t ch)  {

			if (ch < ASCIIL) ch = ASCIIL;  	
			if (ch > ASCIIH) ch = ASCIIH;
	
			int base;
			uint8_t i, p, cmd;
  
			uint8_t tmppage=page;
			uint8_t tmpcol=column;	
			uint8_t EOL;	

			EOL = column + CHAR_FW;	
			base = (ch - ASCIIL) * CHAR_PG * CHAR_FW; 
								
			
			if (EOL > 128){	
			
				column = 0;
				tmpcol = 0;
				cSelect(column); 

				WriteCommand(SET_ADDRESS);
			
				page = page + CHAR_PG;
				tmppage=page;
				cmd = SET_PAGE | page;						
				WriteCommand(cmd);
				
			}
								
			for (p=0;p<CHAR_PG;p++){	
			
				for(i = 0; i < CHAR_FW ;i++){ 
													
	
				cSelect(tmpcol);	
					DataMode();
					dPort = pgm_read_byte(&basicfont[base++]); 
					LocknLoad();
				
					tmpcol++;
				}

				if(CHAR_PG>1){
				   
				  tmppage = tmppage + 1;

				  cmd = SET_PAGE | tmppage;	
				  WriteCommand(cmd);				  
				
				  cmd = SET_ADDRESS | column;
				  WriteCommand(cmd);
				  tmpcol = column;	
				  
				}

			}
			
					
	
			if ((column + CHAR_SPACE) < 128){											
				column = column + CHAR_FW + CHAR_SPACE;	
				cmd = SET_ADDRESS | column;	
				WriteCommand(cmd);												  
			}
						
		
			if(CHAR_PG>1){	  
				cmd = SET_PAGE | page;
				WriteCommand(cmd);			  				  
			}
			
				
}




void printl(const char* str) {
  while(*str) {
    lcd_writech(*str++);
  }
}
