/*****************************************************************************
Author: Peter J. Vis
First Written: 8 Dec 2004
Last Updated: 12 Dec 2006
Title: Displaying text on a 128x64 LCD - Header File

Microcontroller: ATmega32
Crystal: 16Mhz
Platform: Development System

Documentation:
Further information regarding this code is available on my site:
Also check the library file for the Bar graph project. I have
provided a lot of comments on that.
http://petervis.com

PURPOSE:
This is the header file.

LIMITATIONS:
No part of this work may be used in commercial
applications without prior written permission. This work cannot be
used by Bloggers, or any kind of on-line publishing.

COPYRIGHT:	
All the material including function names are copyright protected.

CIRCUIT:
My Modular Development System

*******************************************************************************/


#include <avr/pgmspace.h>
#include <avr/io.h>


// Clock Oscillator.
#define F_CPU 16000000UL

// Control Port.
#define	cPort	 	PORTB
#define	cPort_ddr	DDRB
#define	cPort_pins	PINB

// Data Port.
#define	dPort		PORTD
#define	dPort_ddr	DDRD
#define	dPort_pins	PIND


                        // Control Port Bits used.
						// Function ------------- LCD ------ PortB
#define	lcdrs	0 		// RS ALSO CALLED D/I --- pin4 ----- Pin0 
#define	lcdrw	1 		// R/W ------------------ pin5 ----- Pin1 
#define	lcde	2       // e -------------------- pin6 ----- Pin2 
#define	lcdcs1	5       // CS1 ------------------ pin15 ---- Pin5 
#define	lcdcs2	4       // CS2 ------------------ pin16 ---- Pin4 
#define	lcdrst	3       // RST ------------------ pin17 ---- Pin3 


#define START_LINE		0b11000000
#define SET_ADDRESS		0b01000000
#define SET_PAGE		0b10111000
#define DISP_ON			0b00111111
#define DISP_OFF		0b00111110


unsigned char column = 0;
unsigned char page = 0;


void cSelect(uint8_t col);


void CommandMode (void);
void WriteCommand(uint8_t cmd);


void DataMode (void);
void LocknLoad (void);


void lcdInit (void);
void clearl (void);					

void setNewPage (unsigned char PageData);

// This sets the cursor position.
void setXY(uint8_t x, uint8_t y);


// Font and text display functions.						
void lcd_writech(uint8_t ch); 			// This is my main Character Driver.
void printl(const char* str);			// This is my LCD user printing function.


